@@ -450,7 +450,7 @@ protected:
 	int bracesStyle;
 	int braceCount;
 
-	bool indentationWSVisible;
+	int indentationWSVisible;
 	int indentExamine;
 	bool autoCompleteIgnoreCase;
 	bool imeAutoComplete;
@@ -464,6 +464,7 @@ protected:
 	std::string calltipEndDefinition;
 	std::string autoCompleteStartCharacters;
 	std::string autoCompleteFillUpCharacters;
+	std::string autoCompleteTypeSeparator;
 	std::string wordCharacters;
 	std::string whitespaceCharacters;
 	int startCalltipWord;
@@ -494,6 +495,7 @@ protected:
 	std::string currentMacro;
 	bool recording;
 
+	PropSetFile propsPlatform;
 	PropSetFile propsEmbed;
 	PropSetFile propsBase;
 	PropSetFile propsUser;
@@ -541,6 +543,8 @@ protected:
 	void MoveTabRight();
 	void MoveTabLeft();
 
+	virtual void ReadEmbeddedProperties();
+	void ReadEnvironment();
 	void ReadGlobalPropFile();
 	void ReadAbbrevPropFile();
 	void ReadLocalPropFile();
@@ -699,6 +703,7 @@ protected:
 		mbsIconWarning = 0x30
 	};
 	virtual MessageBoxChoice WindowMessageBox(GUI::Window &w, const GUI::gui_string &msg, MessageBoxStyle style = mbsIconWarning) = 0;
+	void FailedSaveMessageBox(const FilePath &filePathSaving);
 	virtual void FindMessageBox(const std::string &msg, const std::string *findItem = 0) = 0;
 	bool FindReplaceAdvanced() const;
 	int FindInTarget(std::string findWhatText, int startPosition, int endPosition);
@@ -735,7 +740,6 @@ protected:
 	virtual void FindReplace(bool replace) = 0;
 	void OutputAppendString(const char *s, int len = -1);
 	virtual void OutputAppendStringSynchronised(const char *s, int len = -1);
-	void MakeOutputVisible();
 	virtual void Execute();
 	virtual void StopExecute() = 0;
 	void ShowMessages(int line);
@@ -787,8 +791,7 @@ protected:
 	void MenuCommand(int cmdID, int source = 0);
 	void FoldChanged(int line, int levelNow, int levelPrev);
 	void FoldChanged(int position);
-	void Expand(int &line, bool doExpand, bool force = false,
-	        int visLevels = 0, int level = -1);
+	void ExpandFolds(int line, bool expand, int level);
 	void FoldAll();
 	void ToggleFoldRecursive(int line, int level);
 	void EnsureAllChildrenVisible(int line, int level);
@@ -804,13 +807,16 @@ protected:
 	virtual void ActivateWindow(const char *timestamp) = 0;
 
 	void RemoveFindMarks();
+	int SearchFlags(bool regularExpressions) const;
 	void MarkAll(MarkPurpose purpose=markWithBookMarks);
 	void BookmarkAdd(int lineno = -1);
 	void BookmarkDelete(int lineno = -1);
 	bool BookmarkPresent(int lineno = -1);
 	void BookmarkToggle(int lineno = -1);
 	void BookmarkNext(bool forwardScan = true, bool select = false);
-	virtual void ToggleOutputVisible();
+	void SetOutputVisibility(bool show);
+	virtual void ShowOutputOnMainThread();
+	void ToggleOutputVisible();
 	virtual void SizeContentWindows() = 0;
 	virtual void SizeSubWindows() = 0;
 
