@@ -64,21 +64,21 @@ long SciTEKeys::ParseKeyCode(const char *mnemonic) {
 			modsInKey |= SCMOD_ALT;
 
 		if (sKey.length() == 1) {
-			keyval = VkKeyScan(sKey[0]) & 0xFF;
+			keyval = VkKeyScan(sKey.at(0)) & 0xFF;
 		} else if (sKey.length() > 1) {
-			if ((sKey[0] == 'F') && (isdigit(sKey[1]))) {
+			if ((sKey.at(0) == 'F') && (isdigit(sKey.at(1)))) {
 				sKey.erase(0, 1);
 				int fkeyNum = atoi(sKey.c_str());
 				if (fkeyNum >= 1 && fkeyNum <= 12)
 					keyval = fkeyNum - 1 + VK_F1;
-			} else if ((sKey[0] == 'V') && (isdigit(sKey[1]))) {
+			} else if ((sKey.at(0) == 'V') && (isdigit(sKey.at(1)))) {
 				sKey.erase(0, 1);
 				int vkey = atoi(sKey.c_str());
 				if (vkey > 0 && vkey <= 0x7FFF)
 					keyval = vkey;
-			} else if (sKey.find("Keypad") == 0) {
+			} else if (StartsWith(sKey, "Keypad")) {
 				sKey.erase(0, strlen("Keypad"));
-				if (isdigit(sKey[0])) {
+				if ((sKey.length() > 0) && isdigit(sKey.at(0))) {
 					int keyNum = atoi(sKey.c_str());
 					if (keyNum >= 0 && keyNum <= 9)
 						keyval = keyNum + VK_NUMPAD0;
@@ -159,9 +159,9 @@ SciTEWin::SciTEWin(Extension *ext) : SciTEBase(ext) {
 	fontTabs = 0;
 	wFocus = 0;
 
-	memset(&winPlace, 0, sizeof(winPlace));
+	winPlace = WINDOWPLACEMENT();
 	winPlace.length = 0;
-	memset(&rcWorkArea, 0, sizeof(rcWorkArea));
+	rcWorkArea = RECT();
 
 	openWhat[0] = '\0';
 	tooltipText[0] = '\0';
@@ -174,31 +174,14 @@ SciTEWin::SciTEWin(Extension *ext) : SciTEBase(ext) {
 	hWriteSubProcess = NULL;
 	subProcessGroupId = 0;
 
-	// Read properties resource into propsEmbed
-	// The embedded properties file is to allow distributions to be sure
-	// that they have a sensible default configuration even if the properties
-	// files are missing. Also allows a one file distribution of Sc1.EXE.
-	propsEmbed.Clear();
-	// System type properties are also stored in the embedded properties.
-	propsEmbed.Set("PLAT_WIN", "1");
-	propsEmbed.Set("PLAT_WINNT", "1");
-
-	HRSRC handProps = ::FindResource(hInstance, TEXT("Embedded"), TEXT("Properties"));
-	if (handProps) {
-		DWORD size = ::SizeofResource(hInstance, handProps);
-		HGLOBAL hmem = ::LoadResource(hInstance, handProps);
-		if (hmem) {
-			const void *pv = ::LockResource(hmem);
-			if (pv) {
-				propsEmbed.ReadFromMemory(
-				    reinterpret_cast<const char *>(pv), size, FilePath(), filter, NULL, 0);
-			}
-		}
-		::FreeResource(handProps);
-	}
-
 	pathAbbreviations = GetAbbrevPropertiesFileName();
 
+	// System type properties are stored in the platform properties.
+	propsPlatform.Set("PLAT_WIN", "1");
+	propsPlatform.Set("PLAT_WINNT", "1");
+
+	ReadEnvironment();
+
 	ReadGlobalPropFile();
 
 	SetScaleFactor(0);
@@ -369,7 +352,7 @@ void SciTEWin::SetScaleFactor(int scale) {
 		::DeleteDC(hdcMeasure);
 	}
 	std::string sScale = StdStringFromInteger(scale);
-	propsEmbed.Set("ScaleFactor", sScale.c_str());
+	propsPlatform.Set("ScaleFactor", sScale.c_str());
 }
 
 // Allow UTF-8 file names and command lines to be used in calls to io.open and io.popen in Lua scripts.
@@ -399,11 +382,34 @@ FILE *scite_lua_popen(const char *filename, const char *mode) {
 
 }
 
+// Read properties resource into propsEmbed
+// The embedded properties file is to allow distributions to be sure
+// that they have a sensible default configuration even if the properties
+// files are missing. Also allows a one file distribution of Sc1.EXE.
+void SciTEWin::ReadEmbeddedProperties() {
+
+	propsEmbed.Clear();
+
+	HRSRC handProps = ::FindResource(hInstance, TEXT("Embedded"), TEXT("Properties"));
+	if (handProps) {
+		DWORD size = ::SizeofResource(hInstance, handProps);
+		HGLOBAL hmem = ::LoadResource(hInstance, handProps);
+		if (hmem) {
+			const void *pv = ::LockResource(hmem);
+			if (pv) {
+				propsEmbed.ReadFromMemory(
+					static_cast<const char *>(pv), size, FilePath(), filter, NULL, 0);
+			}
+		}
+		::FreeResource(handProps);
+	}
+}
+
 void SciTEWin::ReadPropertiesInitial() {
 	SciTEBase::ReadPropertiesInitial();
 	if (tabMultiLine) {	// Windows specific!
-		long wl = ::GetWindowLong(reinterpret_cast<HWND>(wTabBar.GetID()), GWL_STYLE);
-		::SetWindowLong(reinterpret_cast<HWND>(wTabBar.GetID()), GWL_STYLE, wl | TCS_MULTILINE);
+		long wl = ::GetWindowLong(HwndOf(wTabBar), GWL_STYLE);
+		::SetWindowLong(HwndOf(wTabBar), GWL_STYLE, wl | TCS_MULTILINE);
 	}
 }
 
@@ -437,9 +443,16 @@ FilePath SciTEWin::GetSciteDefaultHome() {
 }
 
 FilePath SciTEWin::GetSciteUserHome() {
-	GUI::gui_char *home = _wgetenv(GUI_TEXT("SciTE_HOME"));
-	if (!home)
-		home = _wgetenv(GUI_TEXT("USERPROFILE"));
+	// First looking for environment variable $SciTE_USERHOME
+	// to set SciteUserHome. If not present we look for $SciTE_HOME
+	// then defaulting to $USERPROFILE
+	GUI::gui_char *home = _wgetenv(GUI_TEXT("SciTE_USERHOME"));
+	if (!home) {
+		home = _wgetenv(GUI_TEXT("SciTE_HOME"));
+		if (!home) {
+			home = _wgetenv(GUI_TEXT("USERPROFILE"));
+		}
+	}
 	return GetSciTEPath(home);
 }
 
@@ -505,7 +518,7 @@ void SciTEWin::ExecuteHelp(const char *cmd) {
 void SciTEWin::CopyAsRTF() {
 	Sci_CharacterRange cr = GetSelection();
 	std::ostringstream oss;
-	SaveToStreamRTF(oss, cr.cpMin, cr.cpMax);
+	SaveToStreamRTF(oss, static_cast<int>(cr.cpMin), static_cast<int>(cr.cpMax));
 	std::string rtf = oss.str();
 	size_t len = rtf.length();
 	HGLOBAL hand = ::GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, len + 1);
@@ -794,8 +807,9 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 	HANDLE hPipeWrite = NULL;
 	HANDLE hPipeRead = NULL;
 	// Create pipe for output redirection
-	// read handle, write handle, security attributes,  number of bytes reserved for pipe - 0 default
-	::CreatePipe(&hPipeRead, &hPipeWrite, &sa, 0);
+	// read handle, write handle, security attributes,  number of bytes reserved for pipe
+	const DWORD pipeBufferSize = 64 * 1024;
+	::CreatePipe(&hPipeRead, &hPipeWrite, &sa, pipeBufferSize);
 
 	// Create pipe for input redirection. In this code, you do not
 	// redirect the output of the child process, but you need a handle
@@ -805,8 +819,8 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 	hWriteSubProcess = NULL;
 	subProcessGroupId = 0;
 	HANDLE hRead2 = NULL;
-	// read handle, write handle, security attributes,  number of bytes reserved for pipe - 0 default
-	::CreatePipe(&hRead2, &hWriteSubProcess, &sa, 0);
+	// read handle, write handle, security attributes,  number of bytes reserved for pipe
+	::CreatePipe(&hRead2, &hWriteSubProcess, &sa, pipeBufferSize);
 
 	::SetHandleInformation(hPipeRead, HANDLE_FLAG_INHERIT, 0);
 	::SetHandleInformation(hWriteSubProcess, HANDLE_FLAG_INHERIT, 0);
@@ -829,9 +843,13 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 
 	PROCESS_INFORMATION pi = {0, 0, 0, 0};
 
+	// Make a mutable copy as the CreateProcess parameter is mutable 
+	const GUI::gui_string sCommand = GUI::StringFromUTF8(jobToRun.command);
+	std::vector<wchar_t> vwcCommand(sCommand.c_str(), sCommand.c_str() + sCommand.length() + 1);
+
 	bool running = ::CreateProcessW(
 			  NULL,
-			  const_cast<wchar_t *>(GUI::StringFromUTF8(jobToRun.command).c_str()),
+			  &vwcCommand[0],
 			  NULL, NULL,
 			  TRUE, CREATE_NEW_PROCESS_GROUP,
 			  NULL,
@@ -845,9 +863,12 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 		std::string runComLine = "cmd.exe /c ";
 		runComLine = runComLine.append(jobToRun.command);
 
+		const GUI::gui_string sRunComLine = GUI::StringFromUTF8(runComLine);
+		std::vector<wchar_t> vwcRunComLine(sRunComLine.c_str(), sRunComLine.c_str() + sRunComLine.length() + 1);
+
 		running = ::CreateProcessW(
 			  NULL,
-			  const_cast<wchar_t*>(GUI::StringFromUTF8(runComLine).c_str()),
+			  &vwcRunComLine[0],
 			  NULL, NULL,
 			  TRUE, CREATE_NEW_PROCESS_GROUP,
 			  NULL,
@@ -880,6 +901,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 		unsigned writingPosition = 0;
 
 		int countPeeks = 0;
+		bool processDead = false;
 		while (running) {
 			if (writingPosition >= totalBytesToWrite) {
 				if (countPeeks > 10)
@@ -889,9 +911,15 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 				countPeeks++;
 			}
 
+			// If we don't already know the process is dead,
+			// check now (before polling the output pipe)
+			if (!processDead && (WAIT_OBJECT_0 == ::WaitForSingleObject(pi.hProcess, 0))) {
+				processDead = true;
+			}
+
 			DWORD bytesRead = 0;
 			DWORD bytesAvail = 0;
-			std::vector<char> buffer(16*1024);
+			std::vector<char> buffer(pipeBufferSize);
 
 			if (!::PeekNamedPipe(hPipeRead, &buffer[0],
 					     static_cast<DWORD>(buffer.size()), &bytesRead, &bytesAvail, NULL)) {
@@ -916,7 +944,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 				DWORD bytesWrote = 0;
 
 				int bTest = ::WriteFile(hWriteSubProcess,
-					    const_cast<char *>(jobToRun.input.c_str() + writingPosition),
+					    jobToRun.input.c_str() + writingPosition,
 					    static_cast<DWORD>(bytesToWrite), &bytesWrote, NULL);
 
 				if (bTest) {
@@ -951,7 +979,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 
 					if (!(jobToRun.flags & jobQuiet)) {
 						if (!cmdWorker.seenOutput) {
-							MakeOutputVisible();
+							ShowOutputOnMainThread();
 							cmdWorker.seenOutput = true;
 						}
 						// Display the data
@@ -962,12 +990,13 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 				} else {
 					running = false;
 				}
-			} else {
-				if (::GetExitCodeProcess(pi.hProcess, &exitcode)) {
-					if (STILL_ACTIVE != exitcode) {
-						// Already dead
-						running = false;
-					}
+			} else { 
+				// bytesAvail == 0, and if the process
+				// was already dead by the time we did
+				// PeekNamedPipe, there should not be
+				// any more data coming	
+				if (processDead) {
+					running = false;
 				}
 			}
 
@@ -1011,7 +1040,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 
 			if (doRepSel) {
 				int cpMin = static_cast<int>(wEditor.Send(SCI_GETSELECTIONSTART, 0, 0));
-				wEditor.Send(SCI_REPLACESEL,0,reinterpret_cast<sptr_t>(repSelBuf.c_str()));
+				wEditor.Send(SCI_REPLACESEL,0,SptrFromString(repSelBuf.c_str()));
 				wEditor.Send(SCI_SETSEL, cpMin, cpMin+repSelBuf.length());
 			}
 		}
@@ -1159,18 +1188,18 @@ void SciTEWin::Execute() {
 	cmdWorker.commandTime.Duration(true);
 	cmdWorker.flags = jobQueue.jobQueue[cmdWorker.icmd].flags;
 	if (scrollOutput)
-		wOutput.Send(SCI_GOTOPOS, wOutput.Send(SCI_GETTEXTLENGTH));
+		wOutput.Call(SCI_GOTOPOS, wOutput.Call(SCI_GETTEXTLENGTH));
 
 	if (jobQueue.jobQueue[cmdWorker.icmd].jobType == jobExtension) {
 		// Execute extensions synchronously
 		if (jobQueue.jobQueue[cmdWorker.icmd].flags & jobGroupUndo)
-			wEditor.Send(SCI_BEGINUNDOACTION);
+			wEditor.Call(SCI_BEGINUNDOACTION);
 
 		if (extender)
 			extender->OnExecute(jobQueue.jobQueue[cmdWorker.icmd].command.c_str());
 
 		if (jobQueue.jobQueue[cmdWorker.icmd].flags & jobGroupUndo)
-			wEditor.Send(SCI_ENDUNDOACTION);
+			wEditor.Call(SCI_ENDUNDOACTION);
 
 		ExecuteNext();
 	} else {
@@ -1234,12 +1263,12 @@ static void WorkerThread(void *ptr) {
 }
 
 bool SciTEWin::PerformOnNewThread(Worker *pWorker) {
-	uintptr_t result = _beginthread(WorkerThread, 1024 * 1024, reinterpret_cast<void *>(pWorker));
+	uintptr_t result = _beginthread(WorkerThread, 1024 * 1024, static_cast<void *>(pWorker));
 	return result != static_cast<uintptr_t>(-1);
 }
 
 void SciTEWin::PostOnMainThread(int cmd, Worker *pWorker) {
-	::PostMessage(reinterpret_cast<HWND>(wSciTE.GetID()), SCITE_WORKER, cmd, reinterpret_cast<LPARAM>(pWorker));
+	::PostMessage(HwndOf(wSciTE), SCITE_WORKER, cmd, reinterpret_cast<LPARAM>(pWorker));
 }
 
 void SciTEWin::WorkerCommand(int cmd, Worker *pWorker) {
@@ -1317,7 +1346,7 @@ void SciTEWin::CreateUI() {
 	             NULL,
 	             NULL,
 	             hInstance,
-	             reinterpret_cast<LPSTR>(this));
+	             this);
 	if (!wSciTE.Created())
 		exit(FALSE);
 
@@ -1649,9 +1678,7 @@ bool SciTEWin::IsStdinBlocked() {
 }
 
 void SciTEWin::MinimizeToTray() {
-	TCHAR n[64] = TEXT("SciTE");
-	NOTIFYICONDATA nid;
-	memset(&nid, 0, sizeof(nid));
+	NOTIFYICONDATA nid = {};
 	nid.cbSize = sizeof(nid);
 	nid.hWnd = MainHWND();
 	nid.uID = 1;
@@ -1659,7 +1686,7 @@ void SciTEWin::MinimizeToTray() {
 	nid.uCallbackMessage = SCITE_TRAY;
 	nid.hIcon = static_cast<HICON>(
 	                ::LoadImage(hInstance, TEXT("SCITE"), IMAGE_ICON, 16, 16, LR_DEFAULTSIZE));
-	StringCopy(nid.szTip, n);
+	StringCopy(nid.szTip, TEXT("SciTE"));
 	::ShowWindow(MainHWND(), SW_MINIMIZE);
 	if (::Shell_NotifyIcon(NIM_ADD, &nid)) {
 		::ShowWindow(MainHWND(), SW_HIDE);
@@ -1667,8 +1694,7 @@ void SciTEWin::MinimizeToTray() {
 }
 
 void SciTEWin::RestoreFromTray() {
-	NOTIFYICONDATA nid;
-	memset(&nid, 0, sizeof(nid));
+	NOTIFYICONDATA nid = {};
 	nid.cbSize = sizeof(nid);
 	nid.hWnd = MainHWND();
 	nid.uID = 1;
@@ -1755,7 +1781,7 @@ LRESULT SciTEWin::KeyUp(WPARAM wParam) {
 
 void SciTEWin::AddToPopUp(const char *label, int cmd, bool enabled) {
 	GUI::gui_string localised = localiser.Text(label);
-	HMENU menu = reinterpret_cast<HMENU>(popup.GetID());
+	HMENU menu = static_cast<HMENU>(popup.GetID());
 	if (0 == localised.length())
 		::AppendMenu(menu, MF_SEPARATOR, 0, TEXT(""));
 	else if (enabled)
@@ -1775,7 +1801,7 @@ LRESULT SciTEWin::ContextMenuMessage(UINT iMessage, WPARAM wParam, LPARAM lParam
 		pt.x = w->Call(SCI_POINTXFROMPOSITION, 0, position);
 		pt.y = w->Call(SCI_POINTYFROMPOSITION, 0, position);
 		POINT spt = {pt.x, pt.y};
-		::ClientToScreen(static_cast<HWND>(w->GetID()), &spt);
+		::ClientToScreen(HwndOf(*w), &spt);
 		pt = GUI::Point(spt.x, spt.y);
 	} else {
 		GUI::Rectangle rcEditor = wEditor.GetPosition();
@@ -1868,6 +1894,10 @@ LRESULT SciTEWin::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 			WorkerCommand(static_cast<int>(wParam), reinterpret_cast<Worker *>(lParam));
 			break;
 
+		case SCITE_SHOWOUTPUT:
+			SetOutputVisibility(true);
+			break;
+
 		case WM_NOTIFY:
 			Notify(reinterpret_cast<SCNotification *>(lParam));
 			break;
@@ -1979,12 +2009,12 @@ LRESULT PASCAL SciTEWin::TWndProc(
     HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam) {
 
 	// Find C++ object associated with window.
-	SciTEWin *scite = reinterpret_cast<SciTEWin *>(PointerFromWindow(hWnd));
+	SciTEWin *scite = static_cast<SciTEWin *>(PointerFromWindow(hWnd));
 	// scite will be zero if WM_CREATE not seen yet
 	if (scite == 0) {
 		if (iMessage == WM_CREATE) {
 			LPCREATESTRUCT cs = reinterpret_cast<LPCREATESTRUCT>(lParam);
-			scite = reinterpret_cast<SciTEWin *>(cs->lpCreateParams);
+			scite = static_cast<SciTEWin *>(cs->lpCreateParams);
 			scite->wSciTE = hWnd;
 			SetWindowPointer(hWnd, scite);
 			return scite->WndProc(iMessage, wParam, lParam);
@@ -2051,9 +2081,9 @@ LRESULT ContentWin::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 	case WM_SETCURSOR:
 		if (ControlIDOfCommand(static_cast<unsigned long>(lParam)) == HTCLIENT) {
 			GUI::Point ptCursor;
-			::GetCursorPos(reinterpret_cast<POINT *>(&ptCursor));
+			::GetCursorPos(PointPointer(&ptCursor));
 			GUI::Point ptClient = ptCursor;
-			::ScreenToClient(pSciTEWin->MainHWND(), reinterpret_cast<POINT *>(&ptClient));
+			::ScreenToClient(pSciTEWin->MainHWND(), PointPointer(&ptClient));
 			GUI::Rectangle rcScintilla = pSciTEWin->wEditor.GetPosition();
 			GUI::Rectangle rcOutput = pSciTEWin->wOutput.GetPosition();
 			if (!rcScintilla.Contains(ptCursor) && !rcOutput.Contains(ptCursor)) {
@@ -2115,7 +2145,7 @@ uptr_t SciTEWin::EventLoop() {
 		if (going) {
 			if (!ModelessHandler(&msg)) {
 				if (!GetID() ||
-					::TranslateAccelerator(reinterpret_cast<HWND>(GetID()), GetAcceleratorTable(), &msg) == 0) {
+					::TranslateAccelerator(static_cast<HWND>(GetID()), GetAcceleratorTable(), &msg) == 0) {
 					::TranslateMessage(&msg);
 					::DispatchMessageW(&msg);
 				}
